@@ -1,4 +1,4 @@
-# Copyright (C) 2018-2022 Rocky Bernstein <rocky@gnu.org>
+# Copyright (C) 2018-2023 Rocky Bernstein <rocky@gnu.org>
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -13,24 +13,26 @@
 #  You should have received a copy of the GNU General Public License
 #  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-from typing import Any, Tuple
-import datetime, py_compile, os, sys
+import datetime
+import os
+import py_compile
+import sys
+from typing import Any, Optional, Tuple
 
 from xdis import iscode
+from xdis.load import load_module
 from xdis.version_info import IS_PYPY, PYTHON_VERSION_TRIPLE, version_tuple_to_str
+
 from uncompyle6.code_fns import check_object_path
-from uncompyle6.semantics import pysource
-from uncompyle6.semantics.pysource import PARSER_DEFAULT_DEBUG
 from uncompyle6.parser import ParserError
+from uncompyle6.semantics import pysource
+from uncompyle6.semantics.fragments import code_deparse as code_deparse_fragments
+from uncompyle6.semantics.linemap import deparse_code_with_map
+from uncompyle6.semantics.pysource import PARSER_DEFAULT_DEBUG, code_deparse
 from uncompyle6.version import __version__
 
 # from uncompyle6.linenumbers import line_number_mapping
 
-from uncompyle6.semantics.pysource import code_deparse
-from uncompyle6.semantics.fragments import code_deparse as code_deparse_fragments
-from uncompyle6.semantics.linemap import deparse_code_with_map
-
-from xdis.load import load_module
 
 def _get_outstream(outfile: str) -> Any:
     dir = os.path.dirname(outfile)
@@ -48,14 +50,14 @@ def decompile(
     co,
     bytecode_version: Tuple[int] = PYTHON_VERSION_TRIPLE,
     out=sys.stdout,
-    showasm=None,
+    showasm: Optional[str]=None,
     showast={},
     timestamp=None,
     showgrammar=False,
     source_encoding=None,
     code_objects={},
     source_size=None,
-    is_pypy=None,
+    is_pypy=False,
     magic_int=None,
     mapstream=None,
     do_fragments=False,
@@ -105,14 +107,11 @@ def decompile(
     if source_size:
         write("# Size of source mod 2**32: %d bytes" % source_size)
 
-    # maybe a second -a will do before as well
-    asm = "after" if showasm else None
-
     grammar = dict(PARSER_DEFAULT_DEBUG)
     if showgrammar:
         grammar["reduce"] = True
 
-    debug_opts = {"asm": asm, "tree": showast, "grammar": grammar}
+    debug_opts = {"asm": showasm, "tree": showast, "grammar": grammar}
 
     try:
         if mapstream:
@@ -145,8 +144,8 @@ def decompile(
                 out,
                 bytecode_version,
                 debug_opts=debug_opts,
-                is_pypy=is_pypy,
                 compile_mode=compile_mode,
+                is_pypy=is_pypy,
             )
             pass
         return deparsed
@@ -188,7 +187,7 @@ def decompile_file(
 
     filename = check_object_path(filename)
     code_objects = {}
-    (version, timestamp, magic_int, co, is_pypy, source_size, sip_hash) = load_module(
+    version, timestamp, magic_int, co, is_pypy, source_size, _ = load_module(
         filename, code_objects
     )
 
@@ -242,7 +241,7 @@ def main(
     compiled_files: list,
     source_files: list,
     outfile=None,
-    showasm=None,
+    showasm: Optional[str] = None,
     showast={},
     do_verify=False,
     showgrammar=False,
@@ -394,14 +393,14 @@ def main(
             try:
                 # FIXME: Something is weird with Pypy here
                 sys.stdout.flush()
-            except:
+            except Exception:
                 pass
     if current_outfile:
         sys.stdout.write("\n")
         try:
             # FIXME: Something is weird with Pypy here
             sys.stdout.flush()
-        except:
+        except Exception:
             pass
         pass
     return (tot_files, okay_files, failed_files, verify_failed_files)
@@ -417,7 +416,6 @@ if sys.platform.startswith("linux") and os.uname()[2][:2] in ["2.", "3.", "4."]:
         mi.close()
         return int(mu) / 1000000
 
-
 else:
 
     def __memUsage():
