@@ -14,13 +14,14 @@ class Python30Parser(Python31Parser):
 
         pt_bp             ::= POP_TOP POP_BLOCK
 
-        assert            ::= assert_expr jmp_true LOAD_ASSERT RAISE_VARARGS_1 COME_FROM POP_TOP
-        assert2           ::= assert_expr jmp_true LOAD_ASSERT expr CALL_FUNCTION_1 RAISE_VARARGS_1
-                              come_froms
+        assert            ::= assert_expr jmp_true LOAD_ASSERT RAISE_VARARGS_1
+                              COME_FROM POP_TOP
+        assert2           ::= assert_expr jmp_true LOAD_ASSERT expr CALL_FUNCTION_1
+                              RAISE_VARARGS_1 come_froms
         call_stmt         ::= expr _come_froms POP_TOP
 
-        return_if_lambda  ::= RETURN_END_IF_LAMBDA COME_FROM POP_TOP
-        compare_chained2  ::= expr COMPARE_OP RETURN_END_IF_LAMBDA
+        return_if_lambda       ::= RETURN_END_IF_LAMBDA COME_FROM POP_TOP
+        compare_chained_right  ::= expr COMPARE_OP RETURN_END_IF_LAMBDA
 
         # FIXME: combine with parse3.2
         whileTruestmt     ::= SETUP_LOOP l_stmts_opt
@@ -73,9 +74,11 @@ class Python30Parser(Python31Parser):
 
         # Need to keep LOAD_FAST as index 1
         set_comp_header  ::= BUILD_SET_0 DUP_TOP STORE_FAST
+
         set_comp_func ::= set_comp_header
-                          LOAD_FAST FOR_ITER store comp_iter
-                          JUMP_BACK POP_TOP JUMP_BACK RETURN_VALUE RETURN_LAST
+                          LOAD_ARG FOR_ITER store comp_iter
+                          JUMP_BACK ending_return
+                          RETURN_VALUE RETURN_LAST
 
         list_comp_header ::= BUILD_LIST_0 DUP_TOP STORE_FAST
         list_comp        ::= list_comp_header
@@ -105,7 +108,7 @@ class Python30Parser(Python31Parser):
         dict_comp_func   ::= BUILD_MAP_0
                              DUP_TOP STORE_FAST
                              LOAD_ARG FOR_ITER store
-                             dict_comp_iter JUMP_BACK RETURN_VALUE RETURN_LAST
+                             dict_comp_iter JUMP_BACK ending_return
 
         stmt         ::= try_except30
         try_except30 ::= SETUP_EXCEPT suite_stmts_opt
@@ -205,12 +208,12 @@ class Python30Parser(Python31Parser):
                            come_froms POP_TOP POP_BLOCK COME_FROM_LOOP
 
 
-        # compare_chained is like x <= y <= z
-        compare_chained1  ::= expr DUP_TOP ROT_THREE COMPARE_OP
-                              jmp_false compare_chained1 _come_froms
-        compare_chained1  ::= expr DUP_TOP ROT_THREE COMPARE_OP
-                              jmp_false compare_chained2 _come_froms
-        compare_chained2 ::= expr COMPARE_OP RETURN_END_IF
+        # A "compare_chained" is two comparisions like x <= y <= z
+        compared_chained_middle  ::= expr DUP_TOP ROT_THREE COMPARE_OP
+                                     jmp_false compared_chained_middle _come_froms
+        compared_chained_middle  ::= expr DUP_TOP ROT_THREE COMPARE_OP
+                                     jmp_false compare_chained_right _come_froms
+        compare_chained_right ::= expr COMPARE_OP RETURN_END_IF
         """
 
 
@@ -270,10 +273,11 @@ class Python30Parser(Python31Parser):
         jmp_true         ::= JUMP_IF_TRUE_OR_POP POP_TOP
         jmp_true         ::= POP_JUMP_IF_TRUE
 
-        compare_chained1 ::= expr DUP_TOP ROT_THREE COMPARE_OP JUMP_IF_FALSE_OR_POP
-                             compare_chained1 COME_FROM
-        compare_chained1 ::= expr DUP_TOP ROT_THREE COMPARE_OP JUMP_IF_FALSE_OR_POP
-                             compare_chained2 COME_FROM
+        compared_chained_middle ::= expr DUP_TOP ROT_THREE COMPARE_OP
+                                    JUMP_IF_FALSE_OR_POP compared_chained_middle
+                                    COME_FROM
+        compared_chained_middle ::= expr DUP_TOP ROT_THREE COMPARE_OP
+                                    JUMP_IF_FALSE_OR_POP compare_chained_right COME_FROM
         ret_or           ::= expr JUMP_IF_TRUE_OR_POP  return_expr_or_cond COME_FROM
         ret_and          ::= expr JUMP_IF_FALSE_OR_POP return_expr_or_cond COME_FROM
         if_exp_ret       ::= expr POP_JUMP_IF_FALSE expr RETURN_END_IF
