@@ -190,10 +190,7 @@ class Python36Parser(Python35Parser):
         tryfinally_return_stmt ::= SETUP_FINALLY suite_stmts_opt POP_BLOCK LOAD_CONST
                                    COME_FROM_FINALLY
 
-        compare_chained2 ::= expr COMPARE_OP come_froms JUMP_FORWARD
-
-        stmt ::= genexpr_func
-        genexpr_func ::= LOAD_ARG _come_froms FOR_ITER store comp_iter JUMP_BACK
+        compare_chained_right ::= expr COMPARE_OP come_froms JUMP_FORWARD
         """
 
     # Some of this is duplicated from parse37. Eventually we'll probably rebase from
