@@ -1,4 +1,4 @@
-#  Copyright (c) 2015-2020, 2022 by Rocky Bernstein
+#  Copyright (c) 2015-2020, 2022-2023 by Rocky Bernstein
 #  Copyright (c) 2005 by Dan Pascu <dan@windowmaker.org>
 #  Copyright (c) 2000-2002 by hartmut Goebel <h.goebel@crazy-compilers.com>
 #
@@ -219,10 +219,20 @@ class Scanner37Base(Scanner):
 
         bytecode = self.build_instructions(co)
 
-        # show_asm = 'both'
         if show_asm in ("both", "before"):
-            for instr in bytecode.get_instructions(co):
-                print(instr.disassemble(self.opc))
+            print("\n# ---- before tokenization:")
+            self.insts = bytecode.disassemble_bytes(
+                co.co_code,
+                varnames=co.co_varnames,
+                names=co.co_names,
+                constants=co.co_consts,
+                cells=bytecode._cell_names,
+                linestarts=bytecode._linestarts,
+                asm_format="extended",
+                filename=co.co_filename,
+                show_source=True,
+                first_line_number=co.co_firstlineno,
+            )
 
         # "customize" is in the process of going away here
         customize = {}
@@ -295,6 +305,8 @@ class Scanner37Base(Scanner):
                         inst.starts_line,
                         inst.is_jump_target,
                         inst.has_extended_arg,
+                        None,
+                        None,
                     )
 
         # Get jump targets
@@ -341,9 +353,9 @@ class Scanner37Base(Scanner):
                     j = tokens_append(
                         j,
                         Token(
-                            come_from_name,
-                            jump_offset,
-                            repr(jump_offset),
+                            opname=come_from_name,
+                            attr=jump_offset,
+                            pattr=repr(jump_offset),
                             offset="%s_%s" % (inst.offset, jump_idx),
                             has_arg=True,
                             opc=self.opc,
@@ -525,6 +537,7 @@ class Scanner37Base(Scanner):
             pass
 
         if show_asm in ("both", "after"):
+            print("\n# ---- after tokenization:")
             for t in tokens:
                 print(t.format(line_prefix=""))
             print()
