@@ -1,4 +1,4 @@
-#  Copyright (c) 2022 by Rocky Bernstein
+#  Copyright (c) 2022-2023 by Rocky Bernstein
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -25,7 +25,7 @@ from uncompyle6.semantics.consts import (
 
 from uncompyle6.parsers.treenode import SyntaxTree
 from uncompyle6.scanners.tok import Token
-from uncompyle6.util import better_repr
+from uncompyle6.util import better_repr, get_code_name
 
 from uncompyle6.semantics.helper import (
     find_code_node,
@@ -146,7 +146,6 @@ class NonterminalActions:
         self.prune()  # stop recursing
 
     def n_classdef(self, node):
-
         if self.version >= (3, 6):
             self.n_classdef36(node)
         elif self.version >= (3, 0):
@@ -159,7 +158,7 @@ class NonterminalActions:
         # * class_name - the name of the class
         # * subclass_info - the parameters to the class  e.g.
         #      class Foo(bar, baz)
-        #             -----------
+        #            ------------
         # * subclass_code - the code for the subclass body
 
         if node == "classdefdeco2":
@@ -181,7 +180,7 @@ class NonterminalActions:
             subclass_code = build_class[-3][1].attr
             class_name = node[0][0].pattr
         else:
-            raise "Internal Error n_classdef: cannot find class name"
+            raise RuntimeError("Internal Error n_classdef: cannot find class name")
 
         if node == "classdefdeco2":
             self.write("\n")
@@ -228,7 +227,10 @@ class NonterminalActions:
         else:
             # from trepan.api import debug; debug()
             raise TypeError(
-                f"Internal Error: n_const_list expects dict, list set, or set; got {lastnodetype}"
+                (
+                    "Internal Error: n_const_list expects dict, list set, or set; got "
+                    f"{lastnodetype}"
+                )
             )
 
         self.indent_more(INDENT_PER_LEVEL)
@@ -267,7 +269,10 @@ class NonterminalActions:
                 if elem == "add_value":
                     elem = elem[0]
                 if elem == "ADD_VALUE":
-                    value = "%r" % elem.pattr
+                    if self.version < (3, 0, 0):
+                        value = "%r" % elem.pattr
+                    else:
+                        value = "%s" % str(elem.pattr)
                 else:
                     assert elem.kind == "ADD_VALUE_VAR"
                     value = "%s" % elem.pattr
@@ -517,7 +522,6 @@ class NonterminalActions:
         self.prune()
 
     def n_docstring(self, node):
-
         indent = self.indent
         doc_node = node[0]
         if doc_node.attr:
@@ -539,7 +543,7 @@ class NonterminalActions:
         self.write(indent)
         docstring = repr(docstring.expandtabs())[1:-1]
 
-        for (orig, replace) in (
+        for orig, replace in (
             ("\\\\", "\t"),
             ("\\r\\n", "\n"),
             ("\\n", "\n"),
@@ -697,8 +701,11 @@ class NonterminalActions:
         self.write("(")
         iter_index = 3
         if self.version > (3, 2):
-            if self.version >= (3, 6):
-                if node[0].kind in ("load_closure", "load_genexpr") and self.version >= (3, 8):
+            if self.version >= (3, 3):
+                if node[0].kind in (
+                    "load_closure",
+                    "load_genexpr",
+                ) and self.version >= (3, 8):
                     code_index = -6
                     is_lambda = self.is_lambda
                     if node[0].kind == "load_genexpr":
@@ -706,13 +713,20 @@ class NonterminalActions:
                     self.closure_walk(node, collection_index=4)
                     self.is_lambda = is_lambda
                 else:
-                    # Python 3.7+ adds optional "come_froms" at node[0] so count from the end
+                    # Python 3.7+ adds optional "come_froms" at node[0] so count from
+                    # the end.
                     if node == "generator_exp_async" and self.version[:2] == (3, 6):
                         code_index = 0
                     else:
                         code_index = -6
-                    iter_index = 4 if self.version < (3, 8) else 3
-                    self.comprehension_walk(node, iter_index=iter_index, code_index=code_index)
+                    iter_index = (
+                        4
+                        if self.version < (3, 8) and not isinstance(node[4], Token)
+                        else 3
+                    )
+                    self.comprehension_walk(
+                        node, iter_index=iter_index, code_index=code_index
+                    )
                     pass
                 pass
         else:
@@ -783,7 +797,7 @@ class NonterminalActions:
     def n_import_from(self, node):
         relative_path_index = 0
         if self.version >= (2, 5):
-            if node[relative_path_index].attr > 0:
+            if node[relative_path_index].pattr > 0:
                 node[2].pattr = ("." * node[relative_path_index].attr) + node[2].pattr
             if self.version > (2, 7):
                 if isinstance(node[1].pattr, tuple):
@@ -1024,10 +1038,9 @@ class NonterminalActions:
         self.prune()
 
     def n_mkfunc(self, node):
-
         code_node = find_code_node(node, -2)
         code = code_node.attr
-        self.write(code.co_name)
+        self.write(get_code_name(code))
         self.indent_more()
 
         self.make_function(node, is_lambda=False, code_node=code_node)
@@ -1072,7 +1085,10 @@ class NonterminalActions:
         else:
             # We can't comment out like above because there may be a trailing ')'
             # that needs to be written
-            assert len(node) == 3 and node[2] in ("RETURN_VALUE_LAMBDA", "LAMBDA_MARKER")
+            assert len(node) == 3 and node[2] in (
+                "RETURN_VALUE_LAMBDA",
+                "LAMBDA_MARKER",
+            )
             self.preorder(node[0])
             self.prune()
 
@@ -1093,7 +1109,12 @@ class NonterminalActions:
         self.write("{")
         if node[0] in ["LOAD_SETCOMP", "LOAD_DICTCOMP"]:
             if self.version == (3, 0):
-                iter_index = 6
+                if len(node) >= 6:
+                    iter_index = 6
+                else:
+                    assert node[1].kind.startswith("MAKE_FUNCTION")
+                    iter_index = 2
+                    pass
             else:
                 iter_index = 1
             self.comprehension_walk_newer(node, iter_index=iter_index, code_index=0)
