@@ -1,4 +1,4 @@
-#  Copyright (c) 2016-2020, 2022 Rocky Bernstein
+#  Copyright (c) 2016-2020, 2023 Rocky Bernstein
 #  Copyright (c) 2005 by Dan Pascu <dan@windowmaker.org>
 #  Copyright (c) 2000-2002 by hartmut Goebel <hartmut@goebel.noris.de>
 
@@ -37,11 +37,12 @@ class Python27Parser(Python2Parser):
         dict_comp ::= LOAD_DICTCOMP MAKE_FUNCTION_0 expr GET_ITER CALL_FUNCTION_1
 
         stmt           ::= dict_comp_func
+
         dict_comp_func ::= BUILD_MAP_0 LOAD_FAST FOR_ITER store
-                           comp_iter JUMP_BACK RETURN_VALUE RETURN_LAST
+                           comp_iter JUMP_BACK ending_return
 
         set_comp_func ::= BUILD_SET_0 LOAD_FAST FOR_ITER store comp_iter
-                          JUMP_BACK RETURN_VALUE RETURN_LAST
+                          JUMP_BACK ending_return
 
         comp_iter     ::= comp_if_not
         comp_if_not   ::= expr jmp_true comp_iter
@@ -115,17 +116,18 @@ class Python27Parser(Python2Parser):
         or         ::= expr_jitop expr COME_FROM
         and        ::= expr JUMP_IF_FALSE_OR_POP expr COME_FROM
 
-        # compare_chained{1,2} is used exclusively in chained_compare
-        compare_chained1 ::= expr DUP_TOP ROT_THREE COMPARE_OP JUMP_IF_FALSE_OR_POP
-                             compare_chained1 COME_FROM
-        compare_chained1 ::= expr DUP_TOP ROT_THREE COMPARE_OP JUMP_IF_FALSE_OR_POP
-                             compare_chained2 COME_FROM
+        # compare_chained{middle,2} is used exclusively in chained_compare
+        compared_chained_middle ::= expr DUP_TOP ROT_THREE COMPARE_OP
+                                    JUMP_IF_FALSE_OR_POP compared_chained_middle
+                                    COME_FROM
+        compared_chained_middle ::= expr DUP_TOP ROT_THREE COMPARE_OP
+                                    JUMP_IF_FALSE_OR_POP compare_chained_right COME_FROM
 
         return_lambda      ::= RETURN_VALUE
         return_lambda      ::= RETURN_VALUE_LAMBDA
 
-        compare_chained2 ::= expr COMPARE_OP return_lambda
-        compare_chained2 ::= expr COMPARE_OP return_lambda
+        compare_chained_right ::= expr COMPARE_OP return_lambda
+        compare_chained_right ::= expr COMPARE_OP return_lambda
 
         # if_exp_true are for conditions which always evaluate true
         # There is dead or non-optional remnants of the condition code though,
@@ -177,11 +179,13 @@ class Python27Parser(Python2Parser):
         while1stmt        ::= SETUP_LOOP returns pb_come_from
         while1stmt        ::= SETUP_LOOP l_stmts_opt JUMP_BACK POP_BLOCK COME_FROM
 
-        whilestmt         ::= SETUP_LOOP testexpr l_stmts_opt JUMP_BACK POP_BLOCK _come_froms
+        whilestmt         ::= SETUP_LOOP testexpr l_stmts_opt JUMP_BACK POP_BLOCK
+                              _come_froms
 
         # Should this be JUMP_BACK+ ?
         # JUMP_BACK should all be to the same location
-        whilestmt         ::= SETUP_LOOP testexpr l_stmts_opt JUMP_BACK JUMP_BACK POP_BLOCK _come_froms
+        whilestmt         ::= SETUP_LOOP testexpr l_stmts_opt JUMP_BACK
+                              JUMP_BACK POP_BLOCK _come_froms
 
         while1elsestmt    ::= SETUP_LOOP l_stmts JUMP_BACK POP_BLOCK
                               else_suitel COME_FROM
