@@ -1,4 +1,4 @@
-#  Copyright (c) 2015-2019, 2021-2022 by Rocky Bernstein
+#  Copyright (c) 2015-2019, 2021-2023 by Rocky Bernstein
 #
 #  This program is free software: you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -63,38 +63,34 @@ The node position 0 will be associated with "import".
 
 # FIXME: DRY code with pysource
 
-from __future__ import print_function
-
 import re
-
-from uncompyle6.semantics import pysource
-from uncompyle6 import parser
-from uncompyle6.scanner import Token, Code, get_scanner
-import uncompyle6.parser as python_parser
-from uncompyle6.semantics.check_ast import checker
-
-from uncompyle6.show import maybe_show_asm, maybe_show_tree
-
-from uncompyle6.parsers.treenode import SyntaxTree
-
-from uncompyle6.semantics.pysource import ParserError, StringIO
-from xdis import iscode
-from xdis.version_info import IS_PYPY, PYTHON_VERSION_TRIPLE
-
-from uncompyle6.semantics.consts import (
-    INDENT_PER_LEVEL,
-    NONE,
-    PRECEDENCE,
-    TABLE_DIRECT,
-    escape,
-    MAP,
-    PASS,
-)
+import sys
+from bisect import bisect_right
+from collections import namedtuple
+from typing import Optional
 
 from spark_parser import DEFAULT_DEBUG as PARSER_DEFAULT_DEBUG
 from spark_parser.ast import GenericASTTraversalPruningException
+from xdis import iscode
+from xdis.version_info import IS_PYPY, PYTHON_VERSION_TRIPLE
 
-from collections import namedtuple
+import uncompyle6.parser as python_parser
+from uncompyle6 import parser
+from uncompyle6.parsers.treenode import SyntaxTree
+from uncompyle6.scanner import Code, Token, get_scanner
+from uncompyle6.semantics import pysource
+from uncompyle6.semantics.check_ast import checker
+from uncompyle6.semantics.consts import (
+    INDENT_PER_LEVEL,
+    MAP,
+    NONE,
+    PASS,
+    PRECEDENCE,
+    TABLE_DIRECT,
+    escape,
+)
+from uncompyle6.semantics.pysource import ParserError, StringIO
+from uncompyle6.show import maybe_show_asm, maybe_show_tree
 
 NodeInfo = namedtuple("NodeInfo", "node start finish")
 ExtractInfo = namedtuple(
@@ -149,7 +145,6 @@ TABLE_DIRECT_FRAGMENT = {
 
 
 class FragmentsWalker(pysource.SourceWalker, object):
-
     MAP_DIRECT_FRAGMENT = ()
 
     stacked_params = ("f", "indent", "is_lambda", "_globals")
@@ -346,7 +341,6 @@ class FragmentsWalker(pysource.SourceWalker, object):
             self.prune()  # stop recursing
 
     def n_return_if_stmt(self, node):
-
         start = len(self.f.getvalue()) + len(self.indent)
         if self.params["is_lambda"]:
             node[0].parent = node
@@ -667,7 +661,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
         assert n == "comp_iter"
         # Find the comprehension body. It is the inner-most
         # node that is not list_.. .
-        while n == "comp_iter":   # list_iter
+        while n == "comp_iter":  # list_iter
             n = n[0]  # recurse one step
             if n == "comp_for":
                 if n[0] == "SETUP_LOOP":
@@ -1123,8 +1117,9 @@ class FragmentsWalker(pysource.SourceWalker, object):
 
     n_classdefdeco2 = n_classdef
 
-    def gen_source(self, ast, name, customize, is_lambda=False, returnNone=False,
-                   debug_opts=None):
+    def gen_source(
+        self, ast, name, customize, is_lambda=False, returnNone=False, debug_opts=None
+    ):
         """convert parse tree to Python source code"""
 
         rn = self.return_none
@@ -1150,7 +1145,6 @@ class FragmentsWalker(pysource.SourceWalker, object):
         noneInNames=False,
         is_top_level_module=False,
     ):
-
         # FIXME: DRY with pysource.py
 
         # assert isinstance(tokens[0], Token)
@@ -1169,11 +1163,14 @@ class FragmentsWalker(pysource.SourceWalker, object):
                 self.p.insts = self.scanner.insts
                 self.p.offset2inst_index = self.scanner.offset2inst_index
                 ast = python_parser.parse(self.p, tokens, customize, code)
+                self.customize(customize)
                 self.p.insts = p_insts
+
             except (python_parser.ParserError, AssertionError) as e:
                 raise ParserError(e, tokens)
+            transform_tree = self.treeTransform.transform(ast, code)
             maybe_show_tree(self, ast)
-            return ast
+            return transform_tree
 
         # The bytecode for the end of the main routine has a
         # "return None". However you can't issue a "return" statement in
@@ -1199,23 +1196,28 @@ class FragmentsWalker(pysource.SourceWalker, object):
         if len(tokens) == 0:
             return PASS
 
-        # Build parse tree from tokenized and massaged disassembly.
+        # Build a parse tree from tokenized and massaged disassembly.
         try:
             # FIXME: have p.insts update in a better way
             # modularity is broken here
             p_insts = self.p.insts
             self.p.insts = self.scanner.insts
             self.p.offset2inst_index = self.scanner.offset2inst_index
+            self.p.opc = self.scanner.opc
             ast = parser.parse(self.p, tokens, customize, code)
             self.p.insts = p_insts
-        except (parser.ParserError, AssertionError) as e:
+        except (python_parser.ParserError, AssertionError) as e:
             raise ParserError(e, tokens, {})
 
-        maybe_show_tree(self, ast)
-
         checker(ast, False, self.ast_errors)
 
-        return ast
+        self.customize(customize)
+        transform_tree = self.treeTransform.transform(ast, code)
+
+        maybe_show_tree(self, ast)
+
+        del ast  # Save memory
+        return transform_tree
 
     # FIXME: we could provide another customized routine
     # that fixes up parents along a particular path to a node that
@@ -1455,7 +1457,6 @@ class FragmentsWalker(pysource.SourceWalker, object):
         self.set_pos_info(node, start, len(self.f.getvalue()))
 
     def print_super_classes3(self, node):
-
         # FIXME: wrap superclasses onto a node
         # as a custom rule
         start = len(self.f.getvalue())
@@ -1474,7 +1475,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
                     # FIXME: this doesn't handle positional and keyword args
                     # properly. Need to do something more like that below
                     # in the non-PYPY 3.6 case.
-                    self.template_engine(('(%[0]{attr}=%c)', 1), node[n-1])
+                    self.template_engine(("(%[0]{attr}=%c)", 1), node[n - 1])
                     return
                 else:
                     kwargs = node[n - 1].attr
@@ -1838,9 +1839,13 @@ class FragmentsWalker(pysource.SourceWalker, object):
 
                 index = entry[arg]
                 if isinstance(index, tuple):
-                    assert node[index[0]] == index[1], (
-                        "at %s[%d], expected %s node; got %s"
-                        % (node.kind, arg, node[index[0]].kind, index[1])
+                    assert (
+                        node[index[0]] == index[1]
+                    ), "at %s[%d], expected %s node; got %s" % (
+                        node.kind,
+                        arg,
+                        node[index[0]].kind,
+                        index[1],
                     )
                     index = index[0]
                 assert isinstance(
@@ -1861,9 +1866,13 @@ class FragmentsWalker(pysource.SourceWalker, object):
                 assert isinstance(tup, tuple)
                 if len(tup) == 3:
                     (index, nonterm_name, self.prec) = tup
-                    assert node[index] == nonterm_name, (
-                        "at %s[%d], expected '%s' node; got '%s'"
-                        % (node.kind, arg, nonterm_name, node[index].kind)
+                    assert (
+                        node[index] == nonterm_name
+                    ), "at %s[%d], expected '%s' node; got '%s'" % (
+                        node.kind,
+                        arg,
+                        nonterm_name,
+                        node[index].kind,
                     )
                 else:
                     assert len(tup) == 2
@@ -1976,6 +1985,7 @@ class FragmentsWalker(pysource.SourceWalker, object):
 #
 DEFAULT_DEBUG_OPTS = {"asm": False, "tree": False, "grammar": False}
 
+
 # This interface is deprecated
 def deparse_code(
     version,
@@ -2066,7 +2076,9 @@ def code_deparse(
     )
 
     is_top_level_module = co.co_name == "<module>"
-    deparsed.ast = deparsed.build_ast(tokens, customize, co, is_top_level_module=is_top_level_module)
+    deparsed.ast = deparsed.build_ast(
+        tokens, customize, co, is_top_level_module=is_top_level_module
+    )
 
     assert deparsed.ast == "stmts", "Should have parsed grammar start"
 
@@ -2076,7 +2088,7 @@ def code_deparse(
     # convert leading '__doc__ = "..." into doc string
     assert deparsed.ast == "stmts"
 
-    (deparsed.mod_globs, nonlocals) = pysource.find_globals_and_nonlocals(
+    (deparsed.mod_globs, _) = pysource.find_globals_and_nonlocals(
         deparsed.ast, set(), set(), co, version
     )
 
@@ -2111,9 +2123,6 @@ def code_deparse(
     return deparsed
 
 
-from bisect import bisect_right
-
-
 def find_gt(a, x):
     "Find leftmost value greater than x"
     i = bisect_right(a, x)
@@ -2127,8 +2136,8 @@ def code_deparse_around_offset(
     offset,
     co,
     out=StringIO(),
-    version=None,
-    is_pypy=None,
+    version: Optional[tuple] = None,
+    is_pypy: bool = False,
     debug_opts=DEFAULT_DEBUG_OPTS,
 ):
     """
@@ -2139,7 +2148,7 @@ def code_deparse_around_offset(
     assert iscode(co)
 
     if version is None:
-        version = sysinfo2float()
+        version = sys.version_info[:3]
     if is_pypy is None:
         is_pypy = IS_PYPY
 
@@ -2192,8 +2201,7 @@ def deparsed_find(tup, deparsed, code):
     """Return a NodeInfo nametuple for a fragment-deparsed `deparsed` at `tup`.
 
     `tup` is a name and offset tuple, `deparsed` is a fragment object
-    and `code` is instruction bytecode.
-"""
+    and `code` is instruction bytecode."""
     nodeInfo = None
     name, last_i = tup
     if not hasattr(deparsed, "offsets"):
